#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#
from typing import Callable, Any

from .__version__ import __version_info__, __version__
import htf.filters.core  # noqa: F401: needed for compiled oser.core

Filter = Callable[..., bool]


def filter_or(*filters: Filter) -> Filter:
    """
    Create a filter that combines all results or values of ``*filters`` using or-logic.

    Args:
        *filters: a tuple of filters.

    Returns:
        callable: a filter that conjuncts all elements in ``*filters``.
    """

    def _filter_or(*args: Any, **kwargs: Any) -> bool:
        for f in filters:
            try:
                if callable(f):
                    if f(*args, **kwargs):
                        return True
                else:
                    if f:
                        return f
            except BaseException as e:
                print(
                    f"Warning: An exception occurred while evaluating "
                    f"filter_or in '{f.__name__}': '{str(e)}'. "
                    f"Args: {str(args)}, kwargs: {str(kwargs)}"
                )
                pass
        return False

    return _filter_or


def filter_xor(*filters: Filter) -> Filter:
    """
    Create a filter that combines all results or values of ``*filters`` using xor-logic.

    Args:
        *filters: a tuple of filters.

    Returns:
        callable: a filter that xors all elements in ``*filters``.
    """

    def _filter_xor(*args: Any, **kwargs: Any) -> bool:
        ret = False

        for f in filters:
            try:
                if callable(f):
                    ret ^= f(*args, **kwargs)
                else:
                    ret ^= f
            except BaseException as e:
                print(
                    f"Warning: An exception occurred while evaluating "
                    f"filter_xor in '{f.__name__}': '{str(e)}'. "
                    f"Args: {str(args)}, kwargs: {str(kwargs)}"
                )
                ret ^= False
                pass
        return ret

    return _filter_xor


def filter_and(*filters: Filter) -> Filter:
    """
    Create a filter that combines all results or values of ``*filters`` using logical and.

    Args:
        *filters: a tuple of filters.

    Returns:
        callable: a filter that combines all elements in ``*filters`` using logical and.
    """

    def _filter_and(*args: Any, **kwargs: Any) -> bool:
        for f in filters:
            try:
                if callable(f):
                    if not f(*args, **kwargs):
                        return False
                else:
                    if not f:
                        return False
            except BaseException as e:
                print(
                    f"Warning: An exception occurred while evaluating "
                    f"filter_and in '{f.__name__}': '{str(e)}'. "
                    f"Args: {str(args)}, kwargs: {str(kwargs)}"
                )
                return False

        return True

    return _filter_and


def filter_nand(*filters: Filter) -> Filter:
    """
    Create a filter that combines all results or values of ``*filters`` using logical nand.

    Args:
        *filters: a tuple of filters.

    Returns:
        callable: a filter that combines all elements in ``*filters`` using logical nand.
    """
    return lambda *args, **kwargs: filter_not(filter_and(*filters))(*args, **kwargs)


def filter_nor(*filters: Filter) -> Filter:
    """
    Create a filter that combines all results or values of ``*filters`` using logical nor.

    Args:
        *filters: a tuple of filters.

    Returns:
        callable: a filter that combines all elements in ``*filters`` using logical nor.
    """
    return lambda *args, **kwargs: filter_not(filter_or(*filters))(*args, **kwargs)


def filter_xnor(*filters: Filter) -> Filter:
    """
    Create a filter that combines all results or values of ``*filters`` using logical xnor.

    Args:
        *filters: a tuple of filters.

    Returns:
        callable: a filter that combines all elements in ``*filters`` using logical xnor.
    """
    return lambda *args, **kwargs: filter_not(filter_xor(*filters))(*args, **kwargs)


def filter_not(filter_method: Filter) -> Filter:
    """
    Create a filter that inverts ``filter_method``'s result or values.

    Args:
        filter_method (callable): the filter that is inverted.

    Returns:
        callable: a filter that inverts ``filter_method``.
    """
    if callable(filter_method):
        return lambda *args, **kwargs: not filter_method(*args, **kwargs)
    else:
        return lambda *args, **kwargs: not filter_method


__all__ = [
    "__version_info__",
    "__version__",
    "filter_or",
    "filter_xor",
    "filter_and",
    "filter_nand",
    "filter_nor",
    "filter_xnor",
    "filter_not",
]
